# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["VersionListResponse", "Metadata"]


class Metadata(BaseModel):
    author_email: Optional[str] = None
    """Email of the user who created the version."""

    author_id: Optional[str] = None
    """Identifier of the user who created the version."""

    created_on: Optional[str] = None
    """When the version was created."""

    has_preview: Optional[bool] = FieldInfo(alias="hasPreview", default=None)
    """Whether the version can be previewed."""

    modified_on: Optional[str] = None
    """When the version was last modified."""

    source: Optional[
        Literal[
            "unknown",
            "api",
            "wrangler",
            "terraform",
            "dash",
            "dash_template",
            "integration",
            "quick_editor",
            "playground",
            "workersci",
        ]
    ] = None
    """The source of the version upload."""


class VersionListResponse(BaseModel):
    id: Optional[str] = None
    """Unique identifier for the version."""

    metadata: Optional[Metadata] = None

    number: Optional[float] = None
    """Sequential version number."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .telemetry_keys_params import TelemetryKeysParams as TelemetryKeysParams
from .telemetry_query_params import TelemetryQueryParams as TelemetryQueryParams
from .destination_list_params import DestinationListParams as DestinationListParams
from .telemetry_keys_response import TelemetryKeysResponse as TelemetryKeysResponse
from .telemetry_values_params import TelemetryValuesParams as TelemetryValuesParams
from .telemetry_query_response import TelemetryQueryResponse as TelemetryQueryResponse
from .destination_create_params import DestinationCreateParams as DestinationCreateParams
from .destination_list_response import DestinationListResponse as DestinationListResponse
from .destination_update_params import DestinationUpdateParams as DestinationUpdateParams
from .telemetry_values_response import TelemetryValuesResponse as TelemetryValuesResponse
from .destination_create_response import DestinationCreateResponse as DestinationCreateResponse
from .destination_delete_response import DestinationDeleteResponse as DestinationDeleteResponse
from .destination_update_response import DestinationUpdateResponse as DestinationUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from .target_param import TargetParam
from ..zones.ssl_param import SSLParam
from ..zones.waf_param import WAFParam
from ..zones.mirage_param import MirageParam
from ..zones.polish_param import PolishParam
from ..zones.cache_level_param import CacheLevelParam
from ..zones.browser_check_param import BrowserCheckParam
from ..zones.rocket_loader_param import RocketLoaderParam
from ..zones.ip_geolocation_param import IPGeolocationParam
from ..zones.security_level_param import SecurityLevelParam
from ..zones.always_use_https_param import AlwaysUseHTTPSParam
from ..zones.browser_cache_ttl_param import BrowserCacheTTLParam
from ..zones.email_obfuscation_param import EmailObfuscationParam
from ..zones.response_buffering_param import ResponseBufferingParam
from ..zones.true_client_ip_header_param import TrueClientIPHeaderParam
from ..zones.automatic_https_rewrites_param import AutomaticHTTPSRewritesParam
from ..zones.opportunistic_encryption_param import OpportunisticEncryptionParam
from ..zones.origin_error_page_pass_thru_param import OriginErrorPagePassThruParam
from ..zones.sort_query_string_for_cache_param import SortQueryStringForCacheParam

__all__ = [
    "PageRuleEditParams",
    "Action",
    "ActionBypassCacheOnCookie",
    "ActionCacheByDeviceType",
    "ActionCacheDeceptionArmor",
    "ActionCacheKeyFields",
    "ActionCacheKeyFieldsValue",
    "ActionCacheKeyFieldsValueCookie",
    "ActionCacheKeyFieldsValueHeader",
    "ActionCacheKeyFieldsValueHost",
    "ActionCacheKeyFieldsValueQueryString",
    "ActionCacheKeyFieldsValueUser",
    "ActionCacheOnCookie",
    "ActionCacheTTLByStatus",
    "ActionDisableApps",
    "ActionDisablePerformance",
    "ActionDisableSecurity",
    "ActionDisableZaraz",
    "ActionEdgeCacheTTL",
    "ActionExplicitCacheControl",
    "ActionForwardingURL",
    "ActionForwardingURLValue",
    "ActionHostHeaderOverride",
    "ActionResolveOverride",
    "ActionRespectStrongEtag",
]


class PageRuleEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    actions: Iterable[Action]
    """The set of actions to perform if the targets of this rule match the request.

    Actions can redirect to another URL or override settings, but not both.
    """

    priority: int
    """
    The priority of the rule, used to define which Page Rule is processed over
    another. A higher number indicates a higher priority. For example, if you have a
    catch-all Page Rule (rule A: `/images/*`) but want a more specific Page Rule to
    take precedence (rule B: `/images/special/*`), specify a higher priority for
    rule B so it overrides rule A.
    """

    status: Literal["active", "disabled"]
    """The status of the Page Rule."""

    targets: Iterable[TargetParam]
    """The rule targets to evaluate on each request."""


class ActionBypassCacheOnCookie(TypedDict, total=False):
    id: Literal["bypass_cache_on_cookie"]
    """
    Bypass cache and fetch resources from the origin server if a regular expression
    matches against a cookie name present in the request.
    """

    value: str
    """The regular expression to use for matching cookie names in the request.

    Refer to
    [Bypass Cache on Cookie setting](https://developers.cloudflare.com/rules/page-rules/reference/additional-reference/#bypass-cache-on-cookie-setting)
    to learn about limited regular expression support.
    """


class ActionCacheByDeviceType(TypedDict, total=False):
    id: Literal["cache_by_device_type"]
    """Separate cached content based on the visitor's device type."""

    value: Literal["on", "off"]
    """The status of Cache By Device Type."""


class ActionCacheDeceptionArmor(TypedDict, total=False):
    id: Literal["cache_deception_armor"]
    """
    Protect from web cache deception attacks while still allowing static assets to
    be cached. This setting verifies that the URL's extension matches the returned
    `Content-Type`.
    """

    value: Literal["on", "off"]
    """The status of Cache Deception Armor."""


class ActionCacheKeyFieldsValueCookie(TypedDict, total=False):
    """Controls which cookies appear in the Cache Key."""

    check_presence: SequenceNotStr[str]
    """
    A list of cookies to check for the presence of, without including their actual
    values.
    """

    include: SequenceNotStr[str]
    """A list of cookies to include."""


class ActionCacheKeyFieldsValueHeader(TypedDict, total=False):
    """Controls which headers go into the Cache Key.

    Exactly one of
    `include` or `exclude` is expected.
    """

    check_presence: SequenceNotStr[str]
    """
    A list of headers to check for the presence of, without including their actual
    values.
    """

    exclude: SequenceNotStr[str]
    """A list of headers to ignore."""

    include: SequenceNotStr[str]
    """A list of headers to include."""


class ActionCacheKeyFieldsValueHost(TypedDict, total=False):
    """Determines which host header to include in the Cache Key."""

    resolved: bool
    """Whether to include the Host header in the HTTP request sent to the origin."""


class ActionCacheKeyFieldsValueQueryString(TypedDict, total=False):
    """Controls which URL query string parameters go into the Cache
    Key.

    Exactly one of `include` or `exclude` is expected.
    """

    exclude: Union[Literal["*"], SequenceNotStr[str]]
    """Ignore all query string parameters."""

    include: Union[Literal["*"], SequenceNotStr[str]]
    """Include all query string parameters."""


class ActionCacheKeyFieldsValueUser(TypedDict, total=False):
    """
    Feature fields to add features about the end-user (client) into
    the Cache Key.
    """

    device_type: bool
    """
    Classifies a request as `mobile`, `desktop`, or `tablet` based on the User
    Agent.
    """

    geo: bool
    """Includes the client's country, derived from the IP address."""

    lang: bool
    """
    Includes the first language code contained in the `Accept-Language` header sent
    by the client.
    """


class ActionCacheKeyFieldsValue(TypedDict, total=False):
    cookie: ActionCacheKeyFieldsValueCookie
    """Controls which cookies appear in the Cache Key."""

    header: ActionCacheKeyFieldsValueHeader
    """Controls which headers go into the Cache Key.

    Exactly one of `include` or `exclude` is expected.
    """

    host: ActionCacheKeyFieldsValueHost
    """Determines which host header to include in the Cache Key."""

    query_string: ActionCacheKeyFieldsValueQueryString
    """Controls which URL query string parameters go into the Cache Key.

    Exactly one of `include` or `exclude` is expected.
    """

    user: ActionCacheKeyFieldsValueUser
    """Feature fields to add features about the end-user (client) into the Cache Key."""


class ActionCacheKeyFields(TypedDict, total=False):
    id: Literal["cache_key_fields"]
    """
    Control specifically what variables to include when deciding which resources to
    cache. This allows customers to determine what to cache based on something other
    than just the URL.
    """

    value: ActionCacheKeyFieldsValue


class ActionCacheOnCookie(TypedDict, total=False):
    id: Literal["cache_on_cookie"]
    """
    Apply the Cache Everything option (Cache Level setting) based on a regular
    expression match against a cookie name.
    """

    value: str
    """The regular expression to use for matching cookie names in the request."""


class ActionCacheTTLByStatus(TypedDict, total=False):
    id: Literal["cache_ttl_by_status"]
    """
    Enterprise customers can set cache time-to-live (TTL) based on the response
    status from the origin web server. Cache TTL refers to the duration of a
    resource in the Cloudflare network before being marked as stale or discarded
    from cache. Status codes are returned by a resource's origin. Setting cache TTL
    based on response status overrides the default cache behavior (standard caching)
    for static files and overrides cache instructions sent by the origin web server.
    To cache non-static assets, set a Cache Level of Cache Everything using a Page
    Rule. Setting no-store Cache-Control or a low TTL (using `max-age`/`s-maxage`)
    increases requests to origin web servers and decreases performance.
    """

    value: Dict[str, Union[Literal["no-cache", "no-store"], int]]
    """
    A JSON object containing status codes and their corresponding TTLs. Each
    key-value pair in the cache TTL by status cache rule has the following syntax

    - `status_code`: An integer value such as 200 or 500. status_code matches the
      exact status code from the origin web server. Valid status codes are between
      100-999.
    - `status_code_range`: Integer values for from and to. status_code_range matches
      any status code from the origin web server within the specified range.
    - `value`: An integer value that defines the duration an asset is valid in
      seconds or one of the following strings: no-store (equivalent to -1), no-cache
      (equivalent to 0).
    """


class ActionDisableApps(TypedDict, total=False):
    id: Literal["disable_apps"]
    """
    Turn off all active
    [Cloudflare Apps](https://developers.cloudflare.com/support/more-dashboard-apps/cloudflare-apps/)
    (deprecated).
    """


class ActionDisablePerformance(TypedDict, total=False):
    id: Literal["disable_performance"]
    """
    Turn off
    [Rocket Loader](https://developers.cloudflare.com/speed/optimization/content/rocket-loader/),
    and [Polish](https://developers.cloudflare.com/images/polish/).
    """


class ActionDisableSecurity(TypedDict, total=False):
    id: Literal["disable_security"]
    """
    Turn off
    [Email Obfuscation](https://developers.cloudflare.com/waf/tools/scrape-shield/email-address-obfuscation/),
    [Rate Limiting (previous version, deprecated)](https://developers.cloudflare.com/waf/reference/legacy/old-rate-limiting/),
    [Scrape Shield](https://developers.cloudflare.com/waf/tools/scrape-shield/),
    [URL (Zone) Lockdown](https://developers.cloudflare.com/waf/tools/zone-lockdown/),
    and
    [WAF managed rules (previous version, deprecated)](https://developers.cloudflare.com/waf/reference/legacy/old-waf-managed-rules/).
    """


class ActionDisableZaraz(TypedDict, total=False):
    id: Literal["disable_zaraz"]
    """Turn off [Zaraz](https://developers.cloudflare.com/zaraz/)."""


class ActionEdgeCacheTTL(TypedDict, total=False):
    id: Literal["edge_cache_ttl"]
    """Specify how long to cache a resource in the Cloudflare global network.

    _Edge Cache TTL_ is not visible in response headers.
    """

    value: int


class ActionExplicitCacheControl(TypedDict, total=False):
    id: Literal["explicit_cache_control"]
    """
    Origin Cache Control is enabled by default for Free, Pro, and Business domains
    and disabled by default for Enterprise domains.
    """

    value: Literal["on", "off"]
    """The status of Origin Cache Control."""


class ActionForwardingURLValue(TypedDict, total=False):
    status_code: Literal[301, 302]
    """The status code to use for the URL redirect.

    301 is a permanent redirect. 302 is a temporary redirect.
    """

    url: str
    """
    The URL to redirect the request to. Notes: ${num} refers to the position of '\\**'
    in the constraint value.
    """


class ActionForwardingURL(TypedDict, total=False):
    id: Literal["forwarding_url"]
    """Redirects one URL to another using an `HTTP 301/302` redirect.

    Refer to
    [Wildcard matching and referencing](https://developers.cloudflare.com/rules/page-rules/reference/wildcard-matching/).
    """

    value: ActionForwardingURLValue


class ActionHostHeaderOverride(TypedDict, total=False):
    id: Literal["host_header_override"]
    """Apply a specific host header."""

    value: str
    """The hostname to use in the `Host` header"""


class ActionResolveOverride(TypedDict, total=False):
    id: Literal["resolve_override"]
    """Change the origin address to the value specified in this setting."""

    value: str
    """The origin address you want to override with."""


class ActionRespectStrongEtag(TypedDict, total=False):
    id: Literal["respect_strong_etag"]
    """
    Turn on or off byte-for-byte equivalency checks between the Cloudflare cache and
    the origin server.
    """

    value: Literal["on", "off"]
    """The status of Respect Strong ETags"""


Action: TypeAlias = Union[
    AlwaysUseHTTPSParam,
    AutomaticHTTPSRewritesParam,
    BrowserCacheTTLParam,
    BrowserCheckParam,
    ActionBypassCacheOnCookie,
    ActionCacheByDeviceType,
    ActionCacheDeceptionArmor,
    ActionCacheKeyFields,
    CacheLevelParam,
    ActionCacheOnCookie,
    ActionCacheTTLByStatus,
    ActionDisableApps,
    ActionDisablePerformance,
    ActionDisableSecurity,
    ActionDisableZaraz,
    ActionEdgeCacheTTL,
    EmailObfuscationParam,
    ActionExplicitCacheControl,
    ActionForwardingURL,
    ActionHostHeaderOverride,
    IPGeolocationParam,
    MirageParam,
    OpportunisticEncryptionParam,
    OriginErrorPagePassThruParam,
    PolishParam,
    ActionResolveOverride,
    ActionRespectStrongEtag,
    ResponseBufferingParam,
    RocketLoaderParam,
    SecurityLevelParam,
    SortQueryStringForCacheParam,
    SSLParam,
    TrueClientIPHeaderParam,
    WAFParam,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["ServiceBinding", "Provisioning"]


class Provisioning(BaseModel):
    """Status of a Service Binding's deployment to the Cloudflare network"""

    state: Optional[Literal["provisioning", "active"]] = None
    """
    When a binding has been deployed to a majority of Cloudflare datacenters, the
    binding will become active and can be used with its associated service.
    """


class ServiceBinding(BaseModel):
    id: Optional[str] = None
    """Identifier of a Service Binding."""

    cidr: Optional[str] = None
    """IP Prefix in Classless Inter-Domain Routing format."""

    provisioning: Optional[Provisioning] = None
    """Status of a Service Binding's deployment to the Cloudflare network"""

    service_id: Optional[str] = None
    """Identifier of a Service on the Cloudflare network.

    Available services and their IDs may be found in the **List Services** endpoint.
    """

    service_name: Optional[str] = None
    """Name of a service running on the Cloudflare network"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["LogoMatchDownloadParams"]


class LogoMatchDownloadParams(TypedDict, total=False):
    account_id: Required[str]

    limit: str

    logo_id: SequenceNotStr[str]

    offset: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["ZeroTrustGroup", "Meta"]


class Meta(BaseModel):
    """The metadata of the SCIM resource."""

    created: Optional[datetime] = None
    """The timestamp of when the SCIM resource was created."""

    last_modified: Optional[datetime] = FieldInfo(alias="lastModified", default=None)
    """The timestamp of when the SCIM resource was last modified."""


class ZeroTrustGroup(BaseModel):
    id: Optional[str] = None
    """The unique Cloudflare-generated Id of the SCIM resource."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)
    """The display name of the SCIM Group resource."""

    external_id: Optional[str] = FieldInfo(alias="externalId", default=None)
    """The IdP-generated Id of the SCIM resource."""

    meta: Optional[Meta] = None
    """The metadata of the SCIM resource."""

    schemas: Optional[List[str]] = None
    """
    The list of URIs which indicate the attributes contained within a SCIM resource.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .bulks import (
    BulksResource,
    AsyncBulksResource,
    BulksResourceWithRawResponse,
    AsyncBulksResourceWithRawResponse,
    BulksResourceWithStreamingResponse,
    AsyncBulksResourceWithStreamingResponse,
)
from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ....types.intel import domain_get_params
from ...._base_client import make_request_options
from ....types.intel.domain import Domain

__all__ = ["DomainsResource", "AsyncDomainsResource"]


class DomainsResource(SyncAPIResource):
    @cached_property
    def bulks(self) -> BulksResource:
        return BulksResource(self._client)

    @cached_property
    def with_raw_response(self) -> DomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DomainsResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        account_id: str,
        domain: str | Omit = omit,
        skip_dns: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Domain]:
        """
        Gets security details and statistics about a domain.

        Args:
          account_id: Identifier.

          skip_dns: Skip DNS resolution lookups for faster response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/domain", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "domain": domain,
                        "skip_dns": skip_dns,
                    },
                    domain_get_params.DomainGetParams,
                ),
                post_parser=ResultWrapper[Optional[Domain]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Domain]], ResultWrapper[Domain]),
        )


class AsyncDomainsResource(AsyncAPIResource):
    @cached_property
    def bulks(self) -> AsyncBulksResource:
        return AsyncBulksResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDomainsResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        account_id: str,
        domain: str | Omit = omit,
        skip_dns: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Domain]:
        """
        Gets security details and statistics about a domain.

        Args:
          account_id: Identifier.

          skip_dns: Skip DNS resolution lookups for faster response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/domain", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "domain": domain,
                        "skip_dns": skip_dns,
                    },
                    domain_get_params.DomainGetParams,
                ),
                post_parser=ResultWrapper[Optional[Domain]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Domain]], ResultWrapper[Domain]),
        )


class DomainsResourceWithRawResponse:
    def __init__(self, domains: DomainsResource) -> None:
        self._domains = domains

        self.get = to_raw_response_wrapper(
            domains.get,
        )

    @cached_property
    def bulks(self) -> BulksResourceWithRawResponse:
        return BulksResourceWithRawResponse(self._domains.bulks)


class AsyncDomainsResourceWithRawResponse:
    def __init__(self, domains: AsyncDomainsResource) -> None:
        self._domains = domains

        self.get = async_to_raw_response_wrapper(
            domains.get,
        )

    @cached_property
    def bulks(self) -> AsyncBulksResourceWithRawResponse:
        return AsyncBulksResourceWithRawResponse(self._domains.bulks)


class DomainsResourceWithStreamingResponse:
    def __init__(self, domains: DomainsResource) -> None:
        self._domains = domains

        self.get = to_streamed_response_wrapper(
            domains.get,
        )

    @cached_property
    def bulks(self) -> BulksResourceWithStreamingResponse:
        return BulksResourceWithStreamingResponse(self._domains.bulks)


class AsyncDomainsResourceWithStreamingResponse:
    def __init__(self, domains: AsyncDomainsResource) -> None:
        self._domains = domains

        self.get = async_to_streamed_response_wrapper(
            domains.get,
        )

    @cached_property
    def bulks(self) -> AsyncBulksResourceWithStreamingResponse:
        return AsyncBulksResourceWithStreamingResponse(self._domains.bulks)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["IPV4Endpoint"]


class IPV4Endpoint(BaseModel):
    enabled: Optional[bool] = None
    """Indicate whether the IPv4 endpoint is enabled for this location."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Type, Iterable, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.browser_rendering import crawl_get_params, crawl_create_params
from ...types.browser_rendering.crawl_get_response import CrawlGetResponse
from ...types.browser_rendering.crawl_create_response import CrawlCreateResponse
from ...types.browser_rendering.crawl_delete_response import CrawlDeleteResponse

__all__ = ["CrawlResource", "AsyncCrawlResource"]


class CrawlResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> CrawlResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return CrawlResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CrawlResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return CrawlResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        account_id: str,
        url: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[crawl_create_params.Variant0AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[crawl_create_params.Variant0AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: crawl_create_params.Variant0Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[crawl_create_params.Variant0Cookie] | Omit = omit,
        crawl_purposes: List[Literal["search", "ai-input", "ai-train"]] | Omit = omit,
        depth: float | Omit = omit,
        emulate_media_type: str | Omit = omit,
        formats: List[Literal["html", "markdown", "json"]] | Omit = omit,
        goto_options: crawl_create_params.Variant0GotoOptions | Omit = omit,
        json_options: crawl_create_params.Variant0JsonOptions | Omit = omit,
        limit: float | Omit = omit,
        max_age: float | Omit = omit,
        modified_since: int | Omit = omit,
        options: crawl_create_params.Variant0Options | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        render: Literal[True] | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        source: Literal["sitemaps", "links", "all"] | Omit = omit,
        viewport: crawl_create_params.Variant0Viewport | Omit = omit,
        wait_for_selector: crawl_create_params.Variant0WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Starts a crawl job for the provided URL and its children.

        Check available
        options like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          url: URL to navigate to, eg. `https://example.com`.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          crawl_purposes: List of crawl purposes to respect Content-Signal directives in robots.txt.
              Allowed values: 'search', 'ai-input', 'ai-train'. Learn more:
              https://contentsignals.org/. Default: ['search', 'ai-input', 'ai-train'].

          depth: Maximum number of levels deep the crawler will traverse from the starting URL.

          formats: Formats to return. Default is `html`.

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          json_options: Options for JSON extraction.

          limit: Maximum number of URLs to crawl.

          max_age: Maximum age of a resource that can be returned from cache in seconds. Default is
              1 day.

          modified_since: Unix timestamp (seconds since epoch) indicating to only crawl pages that were
              modified since this time. For sitemap URLs with a lastmod field, this is
              compared directly. For other URLs, the crawler will use If-Modified-Since header
              when fetching. URLs without modification information (no lastmod in sitemap and
              no Last-Modified header support) will be crawled. Note: This works in
              conjunction with maxAge - both filters must pass for a cached resource to be
              used. Must be within the last year and not in the future.

          options: Additional options for the crawler.

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          render: Whether to render the page or fetch static content. True by default.

          source: Source of links to crawl. 'sitemaps' - only crawl URLs from sitemaps, 'links' -
              only crawl URLs scraped from pages, 'all' - crawl both sitemap and scraped links
              (default).

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        account_id: str,
        render: Literal[False],
        url: str,
        cache_ttl: float | Omit = omit,
        crawl_purposes: List[Literal["search", "ai-input", "ai-train"]] | Omit = omit,
        depth: float | Omit = omit,
        formats: List[Literal["html", "markdown", "json"]] | Omit = omit,
        json_options: crawl_create_params.Variant1JsonOptions | Omit = omit,
        limit: float | Omit = omit,
        max_age: float | Omit = omit,
        modified_since: int | Omit = omit,
        options: crawl_create_params.Variant1Options | Omit = omit,
        source: Literal["sitemaps", "links", "all"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Starts a crawl job for the provided URL and its children.

        Check available
        options like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          render: Whether to render the page or fetch static content. True by default.

          url: URL to navigate to, eg. `https://example.com`.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          crawl_purposes: List of crawl purposes to respect Content-Signal directives in robots.txt.
              Allowed values: 'search', 'ai-input', 'ai-train'. Learn more:
              https://contentsignals.org/. Default: ['search', 'ai-input', 'ai-train'].

          depth: Maximum number of levels deep the crawler will traverse from the starting URL.

          formats: Formats to return. Default is `html`.

          json_options: Options for JSON extraction.

          limit: Maximum number of URLs to crawl.

          max_age: Maximum age of a resource that can be returned from cache in seconds. Default is
              1 day.

          modified_since: Unix timestamp (seconds since epoch) indicating to only crawl pages that were
              modified since this time. For sitemap URLs with a lastmod field, this is
              compared directly. For other URLs, the crawler will use If-Modified-Since header
              when fetching. URLs without modification information (no lastmod in sitemap and
              no Last-Modified header support) will be crawled. Note: This works in
              conjunction with maxAge - both filters must pass for a cached resource to be
              used. Must be within the last year and not in the future.

          options: Additional options for the crawler.

          source: Source of links to crawl. 'sitemaps' - only crawl URLs from sitemaps, 'links' -
              only crawl URLs scraped from pages, 'all' - crawl both sitemap and scraped links
              (default).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "url"], ["account_id", "render", "url"])
    def create(
        self,
        *,
        account_id: str,
        url: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[crawl_create_params.Variant0AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[crawl_create_params.Variant0AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: crawl_create_params.Variant0Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[crawl_create_params.Variant0Cookie] | Omit = omit,
        crawl_purposes: List[Literal["search", "ai-input", "ai-train"]] | Omit = omit,
        depth: float | Omit = omit,
        emulate_media_type: str | Omit = omit,
        formats: List[Literal["html", "markdown", "json"]] | Omit = omit,
        goto_options: crawl_create_params.Variant0GotoOptions | Omit = omit,
        json_options: crawl_create_params.Variant0JsonOptions | crawl_create_params.Variant1JsonOptions | Omit = omit,
        limit: float | Omit = omit,
        max_age: float | Omit = omit,
        modified_since: int | Omit = omit,
        options: crawl_create_params.Variant0Options | crawl_create_params.Variant1Options | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        render: Literal[True] | Literal[False] | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        source: Literal["sitemaps", "links", "all"] | Omit = omit,
        viewport: crawl_create_params.Variant0Viewport | Omit = omit,
        wait_for_selector: crawl_create_params.Variant0WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/browser-rendering/crawl", account_id=account_id),
            body=maybe_transform(
                {
                    "url": url,
                    "action_timeout": action_timeout,
                    "add_script_tag": add_script_tag,
                    "add_style_tag": add_style_tag,
                    "allow_request_pattern": allow_request_pattern,
                    "allow_resource_types": allow_resource_types,
                    "authenticate": authenticate,
                    "best_attempt": best_attempt,
                    "cookies": cookies,
                    "crawl_purposes": crawl_purposes,
                    "depth": depth,
                    "emulate_media_type": emulate_media_type,
                    "formats": formats,
                    "goto_options": goto_options,
                    "json_options": json_options,
                    "limit": limit,
                    "max_age": max_age,
                    "modified_since": modified_since,
                    "options": options,
                    "reject_request_pattern": reject_request_pattern,
                    "reject_resource_types": reject_resource_types,
                    "render": render,
                    "set_extra_http_headers": set_extra_http_headers,
                    "set_java_script_enabled": set_java_script_enabled,
                    "source": source,
                    "viewport": viewport,
                    "wait_for_selector": wait_for_selector,
                    "wait_for_timeout": wait_for_timeout,
                },
                crawl_create_params.CrawlCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"cache_ttl": cache_ttl}, crawl_create_params.CrawlCreateParams),
                post_parser=ResultWrapper[CrawlCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    def delete(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CrawlDeleteResponse:
        """
        Cancels an ongoing crawl job by setting its status to cancelled and stopping all
        queued URLs.

        Args:
          account_id: Account ID.

          job_id: The ID of the crawl job to cancel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/browser-rendering/crawl/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CrawlDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CrawlDeleteResponse], ResultWrapper[CrawlDeleteResponse]),
        )

    def get(
        self,
        job_id: str,
        *,
        account_id: str,
        cache_ttl: float | Omit = omit,
        cursor: float | Omit = omit,
        limit: float | Omit = omit,
        status: Literal["queued", "errored", "completed", "disallowed", "skipped", "cancelled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CrawlGetResponse:
        """
        Returns the result of a crawl job.

        Args:
          account_id: Account ID.

          job_id: Crawl job ID.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          cursor: Cursor for pagination.

          limit: Limit for pagination.

          status: Filter by URL status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/crawl/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cache_ttl": cache_ttl,
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    crawl_get_params.CrawlGetParams,
                ),
                post_parser=ResultWrapper[CrawlGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CrawlGetResponse], ResultWrapper[CrawlGetResponse]),
        )


class AsyncCrawlResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncCrawlResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCrawlResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCrawlResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncCrawlResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        account_id: str,
        url: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[crawl_create_params.Variant0AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[crawl_create_params.Variant0AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: crawl_create_params.Variant0Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[crawl_create_params.Variant0Cookie] | Omit = omit,
        crawl_purposes: List[Literal["search", "ai-input", "ai-train"]] | Omit = omit,
        depth: float | Omit = omit,
        emulate_media_type: str | Omit = omit,
        formats: List[Literal["html", "markdown", "json"]] | Omit = omit,
        goto_options: crawl_create_params.Variant0GotoOptions | Omit = omit,
        json_options: crawl_create_params.Variant0JsonOptions | Omit = omit,
        limit: float | Omit = omit,
        max_age: float | Omit = omit,
        modified_since: int | Omit = omit,
        options: crawl_create_params.Variant0Options | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        render: Literal[True] | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        source: Literal["sitemaps", "links", "all"] | Omit = omit,
        viewport: crawl_create_params.Variant0Viewport | Omit = omit,
        wait_for_selector: crawl_create_params.Variant0WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Starts a crawl job for the provided URL and its children.

        Check available
        options like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          url: URL to navigate to, eg. `https://example.com`.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          action_timeout: The maximum duration allowed for the browser action to complete after the page
              has loaded (such as taking screenshots, extracting content, or generating PDFs).
              If this time limit is exceeded, the action stops and returns a timeout error.

          add_script_tag: Adds a `<script>` tag into the page with the desired URL or content.

          add_style_tag: Adds a `<link rel="stylesheet">` tag into the page with the desired URL or a
              `<style type="text/css">` tag with the content.

          allow_request_pattern: Only allow requests that match the provided regex patterns, eg. '/^.\\**\\..(css)'.

          allow_resource_types: Only allow requests that match the provided resource types, eg. 'image' or
              'script'.

          authenticate: Provide credentials for HTTP authentication.

          best_attempt: Attempt to proceed when 'awaited' events fail or timeout.

          cookies: Check [options](https://pptr.dev/api/puppeteer.page.setcookie).

          crawl_purposes: List of crawl purposes to respect Content-Signal directives in robots.txt.
              Allowed values: 'search', 'ai-input', 'ai-train'. Learn more:
              https://contentsignals.org/. Default: ['search', 'ai-input', 'ai-train'].

          depth: Maximum number of levels deep the crawler will traverse from the starting URL.

          formats: Formats to return. Default is `html`.

          goto_options: Check [options](https://pptr.dev/api/puppeteer.gotooptions).

          json_options: Options for JSON extraction.

          limit: Maximum number of URLs to crawl.

          max_age: Maximum age of a resource that can be returned from cache in seconds. Default is
              1 day.

          modified_since: Unix timestamp (seconds since epoch) indicating to only crawl pages that were
              modified since this time. For sitemap URLs with a lastmod field, this is
              compared directly. For other URLs, the crawler will use If-Modified-Since header
              when fetching. URLs without modification information (no lastmod in sitemap and
              no Last-Modified header support) will be crawled. Note: This works in
              conjunction with maxAge - both filters must pass for a cached resource to be
              used. Must be within the last year and not in the future.

          options: Additional options for the crawler.

          reject_request_pattern: Block undesired requests that match the provided regex patterns, eg.
              '/^.\\**\\..(css)'.

          reject_resource_types: Block undesired requests that match the provided resource types, eg. 'image' or
              'script'.

          render: Whether to render the page or fetch static content. True by default.

          source: Source of links to crawl. 'sitemaps' - only crawl URLs from sitemaps, 'links' -
              only crawl URLs scraped from pages, 'all' - crawl both sitemap and scraped links
              (default).

          viewport: Check [options](https://pptr.dev/api/puppeteer.page.setviewport).

          wait_for_selector: Wait for the selector to appear in page. Check
              [options](https://pptr.dev/api/puppeteer.page.waitforselector).

          wait_for_timeout: Waits for a specified timeout before continuing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        account_id: str,
        render: Literal[False],
        url: str,
        cache_ttl: float | Omit = omit,
        crawl_purposes: List[Literal["search", "ai-input", "ai-train"]] | Omit = omit,
        depth: float | Omit = omit,
        formats: List[Literal["html", "markdown", "json"]] | Omit = omit,
        json_options: crawl_create_params.Variant1JsonOptions | Omit = omit,
        limit: float | Omit = omit,
        max_age: float | Omit = omit,
        modified_since: int | Omit = omit,
        options: crawl_create_params.Variant1Options | Omit = omit,
        source: Literal["sitemaps", "links", "all"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        """Starts a crawl job for the provided URL and its children.

        Check available
        options like `gotoOptions` and `waitFor*` to control page load behaviour.

        Args:
          account_id: Account ID.

          render: Whether to render the page or fetch static content. True by default.

          url: URL to navigate to, eg. `https://example.com`.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          crawl_purposes: List of crawl purposes to respect Content-Signal directives in robots.txt.
              Allowed values: 'search', 'ai-input', 'ai-train'. Learn more:
              https://contentsignals.org/. Default: ['search', 'ai-input', 'ai-train'].

          depth: Maximum number of levels deep the crawler will traverse from the starting URL.

          formats: Formats to return. Default is `html`.

          json_options: Options for JSON extraction.

          limit: Maximum number of URLs to crawl.

          max_age: Maximum age of a resource that can be returned from cache in seconds. Default is
              1 day.

          modified_since: Unix timestamp (seconds since epoch) indicating to only crawl pages that were
              modified since this time. For sitemap URLs with a lastmod field, this is
              compared directly. For other URLs, the crawler will use If-Modified-Since header
              when fetching. URLs without modification information (no lastmod in sitemap and
              no Last-Modified header support) will be crawled. Note: This works in
              conjunction with maxAge - both filters must pass for a cached resource to be
              used. Must be within the last year and not in the future.

          options: Additional options for the crawler.

          source: Source of links to crawl. 'sitemaps' - only crawl URLs from sitemaps, 'links' -
              only crawl URLs scraped from pages, 'all' - crawl both sitemap and scraped links
              (default).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["account_id", "url"], ["account_id", "render", "url"])
    async def create(
        self,
        *,
        account_id: str,
        url: str,
        cache_ttl: float | Omit = omit,
        action_timeout: float | Omit = omit,
        add_script_tag: Iterable[crawl_create_params.Variant0AddScriptTag] | Omit = omit,
        add_style_tag: Iterable[crawl_create_params.Variant0AddStyleTag] | Omit = omit,
        allow_request_pattern: SequenceNotStr[str] | Omit = omit,
        allow_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        authenticate: crawl_create_params.Variant0Authenticate | Omit = omit,
        best_attempt: bool | Omit = omit,
        cookies: Iterable[crawl_create_params.Variant0Cookie] | Omit = omit,
        crawl_purposes: List[Literal["search", "ai-input", "ai-train"]] | Omit = omit,
        depth: float | Omit = omit,
        emulate_media_type: str | Omit = omit,
        formats: List[Literal["html", "markdown", "json"]] | Omit = omit,
        goto_options: crawl_create_params.Variant0GotoOptions | Omit = omit,
        json_options: crawl_create_params.Variant0JsonOptions | crawl_create_params.Variant1JsonOptions | Omit = omit,
        limit: float | Omit = omit,
        max_age: float | Omit = omit,
        modified_since: int | Omit = omit,
        options: crawl_create_params.Variant0Options | crawl_create_params.Variant1Options | Omit = omit,
        reject_request_pattern: SequenceNotStr[str] | Omit = omit,
        reject_resource_types: List[
            Literal[
                "document",
                "stylesheet",
                "image",
                "media",
                "font",
                "script",
                "texttrack",
                "xhr",
                "fetch",
                "prefetch",
                "eventsource",
                "websocket",
                "manifest",
                "signedexchange",
                "ping",
                "cspviolationreport",
                "preflight",
                "other",
            ]
        ]
        | Omit = omit,
        render: Literal[True] | Literal[False] | Omit = omit,
        set_extra_http_headers: Dict[str, str] | Omit = omit,
        set_java_script_enabled: bool | Omit = omit,
        source: Literal["sitemaps", "links", "all"] | Omit = omit,
        viewport: crawl_create_params.Variant0Viewport | Omit = omit,
        wait_for_selector: crawl_create_params.Variant0WaitForSelector | Omit = omit,
        wait_for_timeout: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> str:
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/browser-rendering/crawl", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "url": url,
                    "action_timeout": action_timeout,
                    "add_script_tag": add_script_tag,
                    "add_style_tag": add_style_tag,
                    "allow_request_pattern": allow_request_pattern,
                    "allow_resource_types": allow_resource_types,
                    "authenticate": authenticate,
                    "best_attempt": best_attempt,
                    "cookies": cookies,
                    "crawl_purposes": crawl_purposes,
                    "depth": depth,
                    "emulate_media_type": emulate_media_type,
                    "formats": formats,
                    "goto_options": goto_options,
                    "json_options": json_options,
                    "limit": limit,
                    "max_age": max_age,
                    "modified_since": modified_since,
                    "options": options,
                    "reject_request_pattern": reject_request_pattern,
                    "reject_resource_types": reject_resource_types,
                    "render": render,
                    "set_extra_http_headers": set_extra_http_headers,
                    "set_java_script_enabled": set_java_script_enabled,
                    "source": source,
                    "viewport": viewport,
                    "wait_for_selector": wait_for_selector,
                    "wait_for_timeout": wait_for_timeout,
                },
                crawl_create_params.CrawlCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"cache_ttl": cache_ttl}, crawl_create_params.CrawlCreateParams),
                post_parser=ResultWrapper[CrawlCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[str], ResultWrapper[str]),
        )

    async def delete(
        self,
        job_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CrawlDeleteResponse:
        """
        Cancels an ongoing crawl job by setting its status to cancelled and stopping all
        queued URLs.

        Args:
          account_id: Account ID.

          job_id: The ID of the crawl job to cancel.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/browser-rendering/crawl/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[CrawlDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[CrawlDeleteResponse], ResultWrapper[CrawlDeleteResponse]),
        )

    async def get(
        self,
        job_id: str,
        *,
        account_id: str,
        cache_ttl: float | Omit = omit,
        cursor: float | Omit = omit,
        limit: float | Omit = omit,
        status: Literal["queued", "errored", "completed", "disallowed", "skipped", "cancelled"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CrawlGetResponse:
        """
        Returns the result of a crawl job.

        Args:
          account_id: Account ID.

          job_id: Crawl job ID.

          cache_ttl: Cache TTL default is 5s. Set to 0 to disable.

          cursor: Cursor for pagination.

          limit: Limit for pagination.

          status: Filter by URL status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/browser-rendering/crawl/{job_id}", account_id=account_id, job_id=job_id
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "cache_ttl": cache_ttl,
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    crawl_get_params.CrawlGetParams,
                ),
                post_parser=ResultWrapper[CrawlGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[CrawlGetResponse], ResultWrapper[CrawlGetResponse]),
        )


class CrawlResourceWithRawResponse:
    def __init__(self, crawl: CrawlResource) -> None:
        self._crawl = crawl

        self.create = to_raw_response_wrapper(
            crawl.create,
        )
        self.delete = to_raw_response_wrapper(
            crawl.delete,
        )
        self.get = to_raw_response_wrapper(
            crawl.get,
        )


class AsyncCrawlResourceWithRawResponse:
    def __init__(self, crawl: AsyncCrawlResource) -> None:
        self._crawl = crawl

        self.create = async_to_raw_response_wrapper(
            crawl.create,
        )
        self.delete = async_to_raw_response_wrapper(
            crawl.delete,
        )
        self.get = async_to_raw_response_wrapper(
            crawl.get,
        )


class CrawlResourceWithStreamingResponse:
    def __init__(self, crawl: CrawlResource) -> None:
        self._crawl = crawl

        self.create = to_streamed_response_wrapper(
            crawl.create,
        )
        self.delete = to_streamed_response_wrapper(
            crawl.delete,
        )
        self.get = to_streamed_response_wrapper(
            crawl.get,
        )


class AsyncCrawlResourceWithStreamingResponse:
    def __init__(self, crawl: AsyncCrawlResource) -> None:
        self._crawl = crawl

        self.create = async_to_streamed_response_wrapper(
            crawl.create,
        )
        self.delete = async_to_streamed_response_wrapper(
            crawl.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            crawl.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "MeetingReplaceMeetingByIDParams",
    "AIConfig",
    "AIConfigSummarization",
    "AIConfigTranscription",
    "RecordingConfig",
    "RecordingConfigAudioConfig",
    "RecordingConfigLiveStreamingConfig",
    "RecordingConfigRealtimekitBucketConfig",
    "RecordingConfigStorageConfig",
    "RecordingConfigVideoConfig",
    "RecordingConfigVideoConfigWatermark",
    "RecordingConfigVideoConfigWatermarkSize",
]


class MeetingReplaceMeetingByIDParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    app_id: Required[str]
    """The app identifier tag."""

    ai_config: AIConfig
    """
    The AI Config allows you to customize the behavior of meeting transcriptions and
    summaries
    """

    live_stream_on_start: Optional[bool]
    """Specifies if the meeting should start getting livestreamed on start."""

    persist_chat: bool
    """
    If a meeting is set to persist_chat, meeting chat would remain for a week within
    the meeting space.
    """

    record_on_start: Optional[bool]
    """
    Specifies if the meeting should start getting recorded as soon as someone joins
    the meeting.
    """

    recording_config: RecordingConfig
    """Recording Configurations to be used for this meeting.

    This level of configs takes higher preference over App level configs on the
    RealtimeKit developer portal.
    """

    session_keep_alive_time_in_secs: float
    """
    Time in seconds, for which a session remains active, after the last participant
    has left the meeting.
    """

    summarize_on_end: bool
    """Automatically generate summary of meetings using transcripts.

    Requires Transcriptions to be enabled, and can be retrieved via Webhooks or
    summary API.
    """

    title: Optional[str]
    """Title of the meeting"""


class AIConfigSummarization(TypedDict, total=False):
    """Summary Config"""

    summary_type: Literal[
        "general",
        "team_meeting",
        "sales_call",
        "client_check_in",
        "interview",
        "daily_standup",
        "one_on_one_meeting",
        "lecture",
        "code_review",
    ]
    """Defines the style of the summary, such as general, team meeting, or sales call."""

    text_format: Literal["plain_text", "markdown"]
    """Determines the text format of the summary, such as plain text or markdown."""

    word_limit: int
    """Sets the maximum number of words in the meeting summary."""


class AIConfigTranscription(TypedDict, total=False):
    """Transcription Configurations"""

    keywords: SequenceNotStr[str]
    """Adds specific terms to improve accurate detection during transcription."""

    language: Literal["en-US", "en-IN", "de", "hi", "sv", "ru", "pl", "el", "fr", "nl"]
    """Specifies the language code for transcription to ensure accurate results."""

    profanity_filter: bool
    """Control the inclusion of offensive language in transcriptions."""


class AIConfig(TypedDict, total=False):
    """
    The AI Config allows you to customize the behavior of meeting transcriptions and summaries
    """

    summarization: AIConfigSummarization
    """Summary Config"""

    transcription: AIConfigTranscription
    """Transcription Configurations"""


class RecordingConfigAudioConfig(TypedDict, total=False):
    """Object containing configuration regarding the audio that is being recorded."""

    channel: Literal["mono", "stereo"]
    """Audio signal pathway within an audio file that carries a specific sound source."""

    codec: Literal["MP3", "AAC"]
    """Codec using which the recording will be encoded.

    If VP8/VP9 is selected for videoConfig, changing audioConfig is not allowed. In
    this case, the codec in the audioConfig is automatically set to vorbis.
    """

    export_file: bool
    """Controls whether to export audio file seperately"""


class RecordingConfigLiveStreamingConfig(TypedDict, total=False):
    rtmp_url: str
    """RTMP URL to stream to"""


class RecordingConfigRealtimekitBucketConfig(TypedDict, total=False):
    enabled: Required[bool]
    """Controls whether recordings are uploaded to RealtimeKit's bucket.

    If set to false, `download_url`, `audio_download_url`, `download_url_expiry`
    won't be generated for a recording.
    """


class RecordingConfigStorageConfig(TypedDict, total=False):
    type: Required[Literal["aws", "azure", "digitalocean", "gcs", "sftp"]]
    """Type of storage media."""

    access_key: str
    """Access key of the storage medium.

    Access key is not required for the `gcs` storage media type.

    Note that this field is not readable by clients, only writeable.
    """

    auth_method: Literal["KEY", "PASSWORD"]
    """Authentication method used for "sftp" type storage medium"""

    bucket: str
    """Name of the storage medium's bucket."""

    host: str
    """SSH destination server host for SFTP type storage medium"""

    password: str
    """
    SSH destination server password for SFTP type storage medium when auth_method is
    "PASSWORD". If auth_method is "KEY", this specifies the password for the ssh
    private key.
    """

    path: str
    """Path relative to the bucket root at which the recording will be placed."""

    port: float
    """SSH destination server port for SFTP type storage medium"""

    private_key: str
    """
    Private key used to login to destination SSH server for SFTP type storage
    medium, when auth_method used is "KEY"
    """

    region: str
    """Region of the storage medium."""

    secret: str
    """Secret key of the storage medium.

    Similar to `access_key`, it is only writeable by clients, not readable.
    """

    username: str
    """SSH destination server username for SFTP type storage medium"""


class RecordingConfigVideoConfigWatermarkSize(TypedDict, total=False):
    """Size of the watermark"""

    height: int
    """Height of the watermark in px"""

    width: int
    """Width of the watermark in px"""


class RecordingConfigVideoConfigWatermark(TypedDict, total=False):
    """Watermark to be added to the recording"""

    position: Literal["left top", "right top", "left bottom", "right bottom"]
    """Position of the watermark"""

    size: RecordingConfigVideoConfigWatermarkSize
    """Size of the watermark"""

    url: str
    """URL of the watermark image"""


class RecordingConfigVideoConfig(TypedDict, total=False):
    codec: Literal["H264", "VP8"]
    """Codec using which the recording will be encoded."""

    export_file: bool
    """Controls whether to export video file seperately"""

    height: int
    """Height of the recording video in pixels"""

    watermark: RecordingConfigVideoConfigWatermark
    """Watermark to be added to the recording"""

    width: int
    """Width of the recording video in pixels"""


class RecordingConfig(TypedDict, total=False):
    """Recording Configurations to be used for this meeting.

    This level of configs takes higher preference over App level configs on the RealtimeKit developer portal.
    """

    audio_config: RecordingConfigAudioConfig
    """Object containing configuration regarding the audio that is being recorded."""

    file_name_prefix: str
    """Adds a prefix to the beginning of the file name of the recording."""

    live_streaming_config: RecordingConfigLiveStreamingConfig

    max_seconds: float
    """
    Specifies the maximum duration for recording in seconds, ranging from a minimum
    of 60 seconds to a maximum of 24 hours.
    """

    realtimekit_bucket_config: RecordingConfigRealtimekitBucketConfig

    storage_config: Optional[RecordingConfigStorageConfig]

    video_config: RecordingConfigVideoConfig



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .portal_list_params import PortalListParams as PortalListParams
from .server_list_params import ServerListParams as ServerListParams
from .portal_create_params import PortalCreateParams as PortalCreateParams
from .portal_list_response import PortalListResponse as PortalListResponse
from .portal_read_response import PortalReadResponse as PortalReadResponse
from .portal_update_params import PortalUpdateParams as PortalUpdateParams
from .server_create_params import ServerCreateParams as ServerCreateParams
from .server_list_response import ServerListResponse as ServerListResponse
from .server_read_response import ServerReadResponse as ServerReadResponse
from .server_update_params import ServerUpdateParams as ServerUpdateParams
from .portal_create_response import PortalCreateResponse as PortalCreateResponse
from .portal_delete_response import PortalDeleteResponse as PortalDeleteResponse
from .portal_update_response import PortalUpdateResponse as PortalUpdateResponse
from .server_create_response import ServerCreateResponse as ServerCreateResponse
from .server_delete_response import ServerDeleteResponse as ServerDeleteResponse
from .server_update_response import ServerUpdateResponse as ServerUpdateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.resource_sharing import resource_list_params, resource_create_params, resource_update_params
from ...types.resource_sharing.resource_get_response import ResourceGetResponse
from ...types.resource_sharing.resource_list_response import ResourceListResponse
from ...types.resource_sharing.resource_create_response import ResourceCreateResponse
from ...types.resource_sharing.resource_delete_response import ResourceDeleteResponse
from ...types.resource_sharing.resource_update_response import ResourceUpdateResponse

__all__ = ["ResourcesResource", "AsyncResourcesResource"]


class ResourcesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ResourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ResourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ResourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ResourcesResourceWithStreamingResponse(self)

    def create(
        self,
        share_id: str,
        *,
        account_id: str,
        meta: object,
        resource_account_id: str,
        resource_id: str,
        resource_type: Literal[
            "custom-ruleset",
            "gateway-policy",
            "gateway-destination-ip",
            "gateway-block-page-settings",
            "gateway-extended-email-matching",
            "idp-federation-grant",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceCreateResponse]:
        """
        Adds a resource to an existing share, making it available to share recipients.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          meta: Resource Metadata.

          resource_account_id: Account identifier.

          resource_id: Share Resource identifier.

          resource_type: Resource Type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources", account_id=account_id, share_id=share_id
            ),
            body=maybe_transform(
                {
                    "meta": meta,
                    "resource_account_id": resource_account_id,
                    "resource_id": resource_id,
                    "resource_type": resource_type,
                },
                resource_create_params.ResourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceCreateResponse]], ResultWrapper[ResourceCreateResponse]),
        )

    def update(
        self,
        resource_id: str,
        *,
        account_id: str,
        share_id: str,
        meta: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceUpdateResponse]:
        """
        Update is not immediate, an updated share resource object with a new status will
        be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          resource_id: Share Resource identifier.

          meta: Resource Metadata.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources/{resource_id}",
                account_id=account_id,
                share_id=share_id,
                resource_id=resource_id,
            ),
            body=maybe_transform({"meta": meta}, resource_update_params.ResourceUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceUpdateResponse]], ResultWrapper[ResourceUpdateResponse]),
        )

    def list(
        self,
        share_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        resource_type: Literal[
            "custom-ruleset",
            "gateway-policy",
            "gateway-destination-ip",
            "gateway-block-page-settings",
            "gateway-extended-email-matching",
            "idp-federation-grant",
        ]
        | Omit = omit,
        status: Literal["active", "deleting", "deleted"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[ResourceListResponse]:
        """
        List share resources by share ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          page: Page number.

          per_page: Number of objects to return per page.

          resource_type: Filter share resources by resource_type.

          status: Filter share resources by status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources", account_id=account_id, share_id=share_id
            ),
            page=SyncV4PagePaginationArray[ResourceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "resource_type": resource_type,
                        "status": status,
                    },
                    resource_list_params.ResourceListParams,
                ),
            ),
            model=ResourceListResponse,
        )

    def delete(
        self,
        resource_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceDeleteResponse]:
        """
        Deletion is not immediate, an updated share resource object with a new status
        will be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          resource_id: Share Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources/{resource_id}",
                account_id=account_id,
                share_id=share_id,
                resource_id=resource_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceDeleteResponse]], ResultWrapper[ResourceDeleteResponse]),
        )

    def get(
        self,
        resource_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceGetResponse]:
        """
        Get share resource by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          resource_id: Share Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources/{resource_id}",
                account_id=account_id,
                share_id=share_id,
                resource_id=resource_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceGetResponse]], ResultWrapper[ResourceGetResponse]),
        )


class AsyncResourcesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncResourcesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncResourcesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncResourcesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncResourcesResourceWithStreamingResponse(self)

    async def create(
        self,
        share_id: str,
        *,
        account_id: str,
        meta: object,
        resource_account_id: str,
        resource_id: str,
        resource_type: Literal[
            "custom-ruleset",
            "gateway-policy",
            "gateway-destination-ip",
            "gateway-block-page-settings",
            "gateway-extended-email-matching",
            "idp-federation-grant",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceCreateResponse]:
        """
        Adds a resource to an existing share, making it available to share recipients.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          meta: Resource Metadata.

          resource_account_id: Account identifier.

          resource_id: Share Resource identifier.

          resource_type: Resource Type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources", account_id=account_id, share_id=share_id
            ),
            body=await async_maybe_transform(
                {
                    "meta": meta,
                    "resource_account_id": resource_account_id,
                    "resource_id": resource_id,
                    "resource_type": resource_type,
                },
                resource_create_params.ResourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceCreateResponse]], ResultWrapper[ResourceCreateResponse]),
        )

    async def update(
        self,
        resource_id: str,
        *,
        account_id: str,
        share_id: str,
        meta: object,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceUpdateResponse]:
        """
        Update is not immediate, an updated share resource object with a new status will
        be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          resource_id: Share Resource identifier.

          meta: Resource Metadata.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources/{resource_id}",
                account_id=account_id,
                share_id=share_id,
                resource_id=resource_id,
            ),
            body=await async_maybe_transform({"meta": meta}, resource_update_params.ResourceUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceUpdateResponse]], ResultWrapper[ResourceUpdateResponse]),
        )

    def list(
        self,
        share_id: str,
        *,
        account_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        resource_type: Literal[
            "custom-ruleset",
            "gateway-policy",
            "gateway-destination-ip",
            "gateway-block-page-settings",
            "gateway-extended-email-matching",
            "idp-federation-grant",
        ]
        | Omit = omit,
        status: Literal["active", "deleting", "deleted"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ResourceListResponse, AsyncV4PagePaginationArray[ResourceListResponse]]:
        """
        List share resources by share ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          page: Page number.

          per_page: Number of objects to return per page.

          resource_type: Filter share resources by resource_type.

          status: Filter share resources by status.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        return self._get_api_list(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources", account_id=account_id, share_id=share_id
            ),
            page=AsyncV4PagePaginationArray[ResourceListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "resource_type": resource_type,
                        "status": status,
                    },
                    resource_list_params.ResourceListParams,
                ),
            ),
            model=ResourceListResponse,
        )

    async def delete(
        self,
        resource_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceDeleteResponse]:
        """
        Deletion is not immediate, an updated share resource object with a new status
        will be returned.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          resource_id: Share Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources/{resource_id}",
                account_id=account_id,
                share_id=share_id,
                resource_id=resource_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceDeleteResponse]], ResultWrapper[ResourceDeleteResponse]),
        )

    async def get(
        self,
        resource_id: str,
        *,
        account_id: str,
        share_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[ResourceGetResponse]:
        """
        Get share resource by ID.

        Args:
          account_id: Account identifier.

          share_id: Share identifier tag.

          resource_id: Share Resource identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not share_id:
            raise ValueError(f"Expected a non-empty value for `share_id` but received {share_id!r}")
        if not resource_id:
            raise ValueError(f"Expected a non-empty value for `resource_id` but received {resource_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/shares/{share_id}/resources/{resource_id}",
                account_id=account_id,
                share_id=share_id,
                resource_id=resource_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[ResourceGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[ResourceGetResponse]], ResultWrapper[ResourceGetResponse]),
        )


class ResourcesResourceWithRawResponse:
    def __init__(self, resources: ResourcesResource) -> None:
        self._resources = resources

        self.create = to_raw_response_wrapper(
            resources.create,
        )
        self.update = to_raw_response_wrapper(
            resources.update,
        )
        self.list = to_raw_response_wrapper(
            resources.list,
        )
        self.delete = to_raw_response_wrapper(
            resources.delete,
        )
        self.get = to_raw_response_wrapper(
            resources.get,
        )


class AsyncResourcesResourceWithRawResponse:
    def __init__(self, resources: AsyncResourcesResource) -> None:
        self._resources = resources

        self.create = async_to_raw_response_wrapper(
            resources.create,
        )
        self.update = async_to_raw_response_wrapper(
            resources.update,
        )
        self.list = async_to_raw_response_wrapper(
            resources.list,
        )
        self.delete = async_to_raw_response_wrapper(
            resources.delete,
        )
        self.get = async_to_raw_response_wrapper(
            resources.get,
        )


class ResourcesResourceWithStreamingResponse:
    def __init__(self, resources: ResourcesResource) -> None:
        self._resources = resources

        self.create = to_streamed_response_wrapper(
            resources.create,
        )
        self.update = to_streamed_response_wrapper(
            resources.update,
        )
        self.list = to_streamed_response_wrapper(
            resources.list,
        )
        self.delete = to_streamed_response_wrapper(
            resources.delete,
        )
        self.get = to_streamed_response_wrapper(
            resources.get,
        )


class AsyncResourcesResourceWithStreamingResponse:
    def __init__(self, resources: AsyncResourcesResource) -> None:
        self._resources = resources

        self.create = async_to_streamed_response_wrapper(
            resources.create,
        )
        self.update = async_to_streamed_response_wrapper(
            resources.update,
        )
        self.list = async_to_streamed_response_wrapper(
            resources.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            resources.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            resources.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Hostname"]


class Hostname(BaseModel):
    id: Optional[str] = None
    """Specify the identifier of the hostname."""

    created_on: Optional[datetime] = None

    description: Optional[str] = None
    """Specify an optional description of the hostname."""

    dnslink: Optional[str] = None
    """Specify the DNSLink value used if the target is ipfs."""

    modified_on: Optional[datetime] = None

    name: Optional[str] = None
    """Specify the hostname that points to the target gateway via CNAME."""

    status: Optional[Literal["active", "pending", "deleting", "error"]] = None
    """Specifies the status of the hostname's activation."""

    target: Optional[Literal["ethereum", "ipfs", "ipfs_universal_path"]] = None
    """Specify the target gateway of the hostname."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["DeviceListParams", "LastSeenUser"]


class DeviceListParams(TypedDict, total=False):
    account_id: Required[str]

    id: SequenceNotStr[str]
    """Filter by a one or more device IDs."""

    active_registrations: Literal["include", "only", "exclude"]
    """Include or exclude devices with active registrations.

    The default is "only" - return only devices with active registrations.
    """

    cursor: str
    """
    Opaque token indicating the starting position when requesting the next set of
    records. A cursor value can be obtained from the result_info.cursor field in the
    response.
    """

    include: str
    """
    Comma-separated list of additional information that should be included in the
    device response. Supported values are: "last_seen_registration.policy".
    """

    last_seen_user: LastSeenUser

    per_page: int
    """The maximum number of devices to return in a single response."""

    search: str
    """Search by device details."""

    seen_after: str
    """
    Filter by the last_seen timestamp - returns only devices last seen after this
    timestamp.
    """

    seen_before: str
    """
    Filter by the last_seen timestamp - returns only devices last seen before this
    timestamp.
    """

    sort_by: Literal[
        "name", "id", "client_version", "last_seen_user.email", "last_seen_at", "active_registrations", "created_at"
    ]
    """The device field to order results by."""

    sort_order: Literal["asc", "desc"]
    """Sort direction."""


class LastSeenUser(TypedDict, total=False):
    email: str
    """Filter by the last seen user's email."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["TokenCreateParams", "AccessRule", "Flags"]


class TokenCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    id: str
    """The optional ID of a Stream signing key.

    If present, the `pem` field is also required.
    """

    access_rules: Annotated[Iterable[AccessRule], PropertyInfo(alias="accessRules")]
    """The optional list of access rule constraints on the token.

    Access can be blocked or allowed based on an IP, IP range, or by country. Access
    rules are evaluated from first to last. If a rule matches, the associated action
    is applied and no further rules are evaluated.
    """

    downloadable: bool
    """
    The optional boolean value that enables using signed tokens to access MP4
    download links for a video.
    """

    exp: int
    """
    The optional unix epoch timestamp that specficies the time after a token is not
    accepted. The maximum time specification is 24 hours from issuing time. If this
    field is not set, the default is one hour after issuing.
    """

    flags: Flags
    """Optional flags for the signed token."""

    nbf: int
    """
    The optional unix epoch timestamp that specifies the time before a the token is
    not accepted. If this field is not set, the default is one hour before issuing.
    """

    pem: str
    """
    The optional base64 encoded private key in PEM format associated with a Stream
    signing key. If present, the `id` field is also required.
    """


class AccessRule(TypedDict, total=False):
    """Defines rules for fine-grained control over content than signed URL tokens alone.

    Access rules primarily make tokens conditionally valid based on user information. Access Rules are specified on token payloads as the `accessRules` property containing an array of Rule objects.
    """

    action: Literal["allow", "block"]
    """The action to take when a request matches a rule.

    If the action is `block`, the signed token blocks views for viewers matching the
    rule.
    """

    country: SequenceNotStr[str]
    """
    An array of 2-letter country codes in ISO 3166-1 Alpha-2 format used to match
    requests.
    """

    ip: SequenceNotStr[str]
    """An array of IPv4 or IPV6 addresses or CIDRs used to match requests."""

    type: Literal["any", "ip.src", "ip.geoip.country"]
    """Lists available rule types to match for requests.

    An `any` type matches all requests and can be used as a wildcard to apply
    default actions after other rules.
    """


class Flags(TypedDict, total=False):
    """Optional flags for the signed token."""

    original: bool
    """Whether to return the original video without transformations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ...._utils import PropertyInfo
from ...._models import BaseModel

__all__ = [
    "EventGetResponse",
    "E",
    "EInit",
    "ELeave",
    "EStartAttestation",
    "EFinishAttestationSuccess",
    "EFinishAttestationFailure",
    "EStartRotateCryptKey",
    "EFinishRotateCryptKeySuccess",
    "EFinishRotateCryptKeyFailure",
    "EStartRotatePki",
    "EFinishRotatePkiSuccess",
    "EFinishRotatePkiFailure",
    "EStartUpgrade",
    "EFinishUpgradeSuccess",
    "EFinishUpgradeFailure",
    "EReconcile",
    "EConfigureCloudflaredTunnel",
]


class EInit(BaseModel):
    k: Literal["Init"]
    """Initialized process"""


class ELeave(BaseModel):
    k: Literal["Leave"]
    """Stopped process"""


class EStartAttestation(BaseModel):
    k: Literal["StartAttestation"]
    """Started attestation"""


class EFinishAttestationSuccess(BaseModel):
    k: Literal["FinishAttestationSuccess"]
    """Finished attestation"""


class EFinishAttestationFailure(BaseModel):
    k: Literal["FinishAttestationFailure"]
    """Failed attestation"""


class EStartRotateCryptKey(BaseModel):
    k: Literal["StartRotateCryptKey"]
    """Started crypt key rotation"""


class EFinishRotateCryptKeySuccess(BaseModel):
    k: Literal["FinishRotateCryptKeySuccess"]
    """Finished crypt key rotation"""


class EFinishRotateCryptKeyFailure(BaseModel):
    k: Literal["FinishRotateCryptKeyFailure"]
    """Failed crypt key rotation"""


class EStartRotatePki(BaseModel):
    k: Literal["StartRotatePki"]
    """Started PKI rotation"""


class EFinishRotatePkiSuccess(BaseModel):
    k: Literal["FinishRotatePkiSuccess"]
    """Finished PKI rotation"""


class EFinishRotatePkiFailure(BaseModel):
    k: Literal["FinishRotatePkiFailure"]
    """Failed PKI rotation"""


class EStartUpgrade(BaseModel):
    k: Literal["StartUpgrade"]
    """Started upgrade"""

    url: str
    """Location of upgrade bundle"""


class EFinishUpgradeSuccess(BaseModel):
    k: Literal["FinishUpgradeSuccess"]
    """Finished upgrade"""


class EFinishUpgradeFailure(BaseModel):
    k: Literal["FinishUpgradeFailure"]
    """Failed upgrade"""


class EReconcile(BaseModel):
    k: Literal["Reconcile"]
    """Reconciled"""


class EConfigureCloudflaredTunnel(BaseModel):
    k: Literal["ConfigureCloudflaredTunnel"]
    """Configured Cloudflared tunnel"""


E: TypeAlias = Annotated[
    Union[
        EInit,
        ELeave,
        EStartAttestation,
        EFinishAttestationSuccess,
        EFinishAttestationFailure,
        EStartRotateCryptKey,
        EFinishRotateCryptKeySuccess,
        EFinishRotateCryptKeyFailure,
        EStartRotatePki,
        EFinishRotatePkiSuccess,
        EFinishRotatePkiFailure,
        EStartUpgrade,
        EFinishUpgradeSuccess,
        EFinishUpgradeFailure,
        EReconcile,
        EConfigureCloudflaredTunnel,
    ],
    PropertyInfo(discriminator="k"),
]


class EventGetResponse(BaseModel):
    """Recorded Event"""

    e: E

    n: float
    """Sequence number, used to order events with the same timestamp"""

    t: float
    """Time the Event was recorded (seconds since the Unix epoch)"""

    v: Optional[str] = None
    """Version"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel
from .site_location import SiteLocation

__all__ = ["Site"]


class Site(BaseModel):
    id: Optional[str] = None
    """Identifier"""

    connector_id: Optional[str] = None
    """Magic Connector identifier tag."""

    description: Optional[str] = None

    ha_mode: Optional[bool] = None
    """Site high availability mode.

    If set to true, the site can have two connectors and runs in high availability
    mode.
    """

    location: Optional[SiteLocation] = None
    """Location of site in latitude and longitude."""

    name: Optional[str] = None
    """The name of the site."""

    secondary_connector_id: Optional[str] = None
    """Magic Connector identifier tag. Used when high availability mode is on."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Type, Union, Iterable, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.email_sending import email_sending_send_params, email_sending_send_raw_params
from .subdomains.subdomains import (
    SubdomainsResource,
    AsyncSubdomainsResource,
    SubdomainsResourceWithRawResponse,
    AsyncSubdomainsResourceWithRawResponse,
    SubdomainsResourceWithStreamingResponse,
    AsyncSubdomainsResourceWithStreamingResponse,
)
from ...types.email_sending.email_sending_send_response import EmailSendingSendResponse
from ...types.email_sending.email_sending_send_raw_response import EmailSendingSendRawResponse

__all__ = ["EmailSendingResource", "AsyncEmailSendingResource"]


class EmailSendingResource(SyncAPIResource):
    @cached_property
    def subdomains(self) -> SubdomainsResource:
        return SubdomainsResource(self._client)

    @cached_property
    def with_raw_response(self) -> EmailSendingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return EmailSendingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> EmailSendingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return EmailSendingResourceWithStreamingResponse(self)

    def send(
        self,
        *,
        account_id: str,
        from_: email_sending_send_params.From,
        subject: str,
        to: Union[str, SequenceNotStr[str]],
        attachments: Iterable[email_sending_send_params.Attachment] | Omit = omit,
        bcc: Union[str, SequenceNotStr[str]] | Omit = omit,
        cc: Union[str, SequenceNotStr[str]] | Omit = omit,
        headers: Dict[str, str] | Omit = omit,
        html: str | Omit = omit,
        reply_to: email_sending_send_params.ReplyTo | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmailSendingSendResponse:
        """
        Send an email

        Args:
          account_id: Identifier of the account.

          from_: Sender email address. Either a plain string or an object with address and name.

          subject: Email subject line.

          to: Recipient(s). A single email string or an array of email strings.

          attachments: File attachments and inline images.

          bcc: BCC recipient(s). A single email string or an array of email strings.

          cc: CC recipient(s). A single email string or an array of email strings.

          headers: Custom email headers as key-value pairs.

          html: HTML body of the email. At least one of text or html must be provided.

          reply_to: Reply-to address. Either a plain string or an object with address and name.

          text: Plain text body of the email. At least one of text or html must be provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email/sending/send", account_id=account_id),
            body=maybe_transform(
                {
                    "from_": from_,
                    "subject": subject,
                    "to": to,
                    "attachments": attachments,
                    "bcc": bcc,
                    "cc": cc,
                    "headers": headers,
                    "html": html,
                    "reply_to": reply_to,
                    "text": text,
                },
                email_sending_send_params.EmailSendingSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EmailSendingSendResponse]._unwrapper,
            ),
            cast_to=cast(Type[EmailSendingSendResponse], ResultWrapper[EmailSendingSendResponse]),
        )

    def send_raw(
        self,
        *,
        account_id: str,
        from_: str,
        mime_message: str,
        recipients: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmailSendingSendRawResponse:
        """
        Send a raw MIME email

        Args:
          account_id: Identifier of the account.

          from_: Sender email address.

          mime_message: The full MIME-encoded email message. Should include standard RFC 5322 headers
              such as From, To, Subject, and Content-Type. The from and recipients fields in
              the request body control SMTP envelope routing; the From and To headers in the
              MIME message control what the recipient's email client displays.

          recipients: List of recipient email addresses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/email/sending/send_raw", account_id=account_id),
            body=maybe_transform(
                {
                    "from_": from_,
                    "mime_message": mime_message,
                    "recipients": recipients,
                },
                email_sending_send_raw_params.EmailSendingSendRawParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EmailSendingSendRawResponse]._unwrapper,
            ),
            cast_to=cast(Type[EmailSendingSendRawResponse], ResultWrapper[EmailSendingSendRawResponse]),
        )


class AsyncEmailSendingResource(AsyncAPIResource):
    @cached_property
    def subdomains(self) -> AsyncSubdomainsResource:
        return AsyncSubdomainsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncEmailSendingResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncEmailSendingResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncEmailSendingResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncEmailSendingResourceWithStreamingResponse(self)

    async def send(
        self,
        *,
        account_id: str,
        from_: email_sending_send_params.From,
        subject: str,
        to: Union[str, SequenceNotStr[str]],
        attachments: Iterable[email_sending_send_params.Attachment] | Omit = omit,
        bcc: Union[str, SequenceNotStr[str]] | Omit = omit,
        cc: Union[str, SequenceNotStr[str]] | Omit = omit,
        headers: Dict[str, str] | Omit = omit,
        html: str | Omit = omit,
        reply_to: email_sending_send_params.ReplyTo | Omit = omit,
        text: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmailSendingSendResponse:
        """
        Send an email

        Args:
          account_id: Identifier of the account.

          from_: Sender email address. Either a plain string or an object with address and name.

          subject: Email subject line.

          to: Recipient(s). A single email string or an array of email strings.

          attachments: File attachments and inline images.

          bcc: BCC recipient(s). A single email string or an array of email strings.

          cc: CC recipient(s). A single email string or an array of email strings.

          headers: Custom email headers as key-value pairs.

          html: HTML body of the email. At least one of text or html must be provided.

          reply_to: Reply-to address. Either a plain string or an object with address and name.

          text: Plain text body of the email. At least one of text or html must be provided.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email/sending/send", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "from_": from_,
                    "subject": subject,
                    "to": to,
                    "attachments": attachments,
                    "bcc": bcc,
                    "cc": cc,
                    "headers": headers,
                    "html": html,
                    "reply_to": reply_to,
                    "text": text,
                },
                email_sending_send_params.EmailSendingSendParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EmailSendingSendResponse]._unwrapper,
            ),
            cast_to=cast(Type[EmailSendingSendResponse], ResultWrapper[EmailSendingSendResponse]),
        )

    async def send_raw(
        self,
        *,
        account_id: str,
        from_: str,
        mime_message: str,
        recipients: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> EmailSendingSendRawResponse:
        """
        Send a raw MIME email

        Args:
          account_id: Identifier of the account.

          from_: Sender email address.

          mime_message: The full MIME-encoded email message. Should include standard RFC 5322 headers
              such as From, To, Subject, and Content-Type. The from and recipients fields in
              the request body control SMTP envelope routing; the From and To headers in the
              MIME message control what the recipient's email client displays.

          recipients: List of recipient email addresses.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/email/sending/send_raw", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "from_": from_,
                    "mime_message": mime_message,
                    "recipients": recipients,
                },
                email_sending_send_raw_params.EmailSendingSendRawParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[EmailSendingSendRawResponse]._unwrapper,
            ),
            cast_to=cast(Type[EmailSendingSendRawResponse], ResultWrapper[EmailSendingSendRawResponse]),
        )


class EmailSendingResourceWithRawResponse:
    def __init__(self, email_sending: EmailSendingResource) -> None:
        self._email_sending = email_sending

        self.send = to_raw_response_wrapper(
            email_sending.send,
        )
        self.send_raw = to_raw_response_wrapper(
            email_sending.send_raw,
        )

    @cached_property
    def subdomains(self) -> SubdomainsResourceWithRawResponse:
        return SubdomainsResourceWithRawResponse(self._email_sending.subdomains)


class AsyncEmailSendingResourceWithRawResponse:
    def __init__(self, email_sending: AsyncEmailSendingResource) -> None:
        self._email_sending = email_sending

        self.send = async_to_raw_response_wrapper(
            email_sending.send,
        )
        self.send_raw = async_to_raw_response_wrapper(
            email_sending.send_raw,
        )

    @cached_property
    def subdomains(self) -> AsyncSubdomainsResourceWithRawResponse:
        return AsyncSubdomainsResourceWithRawResponse(self._email_sending.subdomains)


class EmailSendingResourceWithStreamingResponse:
    def __init__(self, email_sending: EmailSendingResource) -> None:
        self._email_sending = email_sending

        self.send = to_streamed_response_wrapper(
            email_sending.send,
        )
        self.send_raw = to_streamed_response_wrapper(
            email_sending.send_raw,
        )

    @cached_property
    def subdomains(self) -> SubdomainsResourceWithStreamingResponse:
        return SubdomainsResourceWithStreamingResponse(self._email_sending.subdomains)


class AsyncEmailSendingResourceWithStreamingResponse:
    def __init__(self, email_sending: AsyncEmailSendingResource) -> None:
        self._email_sending = email_sending

        self.send = async_to_streamed_response_wrapper(
            email_sending.send,
        )
        self.send_raw = async_to_streamed_response_wrapper(
            email_sending.send_raw,
        )

    @cached_property
    def subdomains(self) -> AsyncSubdomainsResourceWithStreamingResponse:
        return AsyncSubdomainsResourceWithStreamingResponse(self._email_sending.subdomains)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PacfileUpdateParams"]


class PacfileUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    contents: Required[str]
    """Actual contents of the PAC file"""

    description: Required[str]
    """Detailed description of the PAC file."""

    name: Required[str]
    """Name of the PAC file."""
